//----------------------------------------------------------------------------
// C main line
//----------------------------------------------------------------------------

#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules
#include <stdlib.h>	
#include <string.h>

const BYTE nameStr[] = "PSoC USBUART Demo";

BYTE theStr[9] = "";   // Define RAM string  
BYTE Len, LCDPos;
BYTE bReceivedData[32];
DWORD dwDTERate;
BOOL oldDTR;

void main()
{
  LCD_Start();                  // Initialize LCD  
  LCDPos = 0;
  oldDTR = FALSE;

  M8C_EnableGInt;                     //Enable Global Interrupts  
  USBUART_Start(  0x03);   //Start USBUART Operation with 5V operation  
  while(!USBUART_Init());  		//Wait for Device to enumerate  

  for(;;)
   {
//    print data about connection settings
    LCD_Position(0,0);            
    USBUART_dwGetDTERate(&dwDTERate);
    ltoa(theStr,dwDTERate,10);
    LCD_PrCString("      ");      // Clear Placeholder
    LCD_Position(0,0);            
    LCD_PrString(theStr);         // Print "PSoC LCD" on the LCD  
    LCD_Position(0,7);            
    itoa(theStr,USBUART_bGetCharFormat(),10);
    LCD_PrString(theStr);         // print count of Stop bits (0 = 1 stop bit, 1 = 1.5 stop bit, 2 = 2 stop bit)
    LCD_Position(0,9);            
    itoa(theStr,USBUART_bGetParityType(),10);
    LCD_PrString(theStr);         // print Parity Type (0 = none, 1 = odd, 2 = even, 3 = mark, 4 = space)
    LCD_Position(0,11);            
    itoa(theStr,USBUART_bGetDataBits(),10);
    LCD_PrString(theStr);         // print count of Data bits
    cstrcpy(theStr,"   ");
    if (USBUART_bGetLineControlBitmap() & 0x01) theStr[0]='D'; else theStr[0]='-';
    if (USBUART_bGetLineControlBitmap() & 0x02) theStr[2]='R'; else theStr[2]='-';
    LCD_Position(0,13);            
    LCD_PrString(theStr);         // print DTR and RTS status

// if the DTR is detected for first time print parameters to the UART
	if ((USBUART_bGetLineControlBitmap() & 0x01) != oldDTR) {	// if DTR changed
	  if (USBUART_bGetLineControlBitmap() & 0x01) { //if DTR is set now
        while (!(USBUART_bTxIsReady()));
        USBUART_CPutString(nameStr);
        while (!(USBUART_bTxIsReady()));
        USBUART_ReadAll(bReceivedData); 
        while (!(USBUART_bTxIsReady()));
        USBUART_PutChar('!');
        while (!(USBUART_bTxIsReady()));
        USBUART_PutCRLF();
        while (!(USBUART_bTxIsReady()));
        USBUART_PutSHexByte(25);
        while (!(USBUART_bTxIsReady()));
        USBUART_PutChar('-');
        while (!(USBUART_bTxIsReady()));
        USBUART_PutSHexInt(35);
      
	  }
	  oldDTR = USBUART_bGetLineControlBitmap() & 0x01;
	}

    //Write back to PC previously received data (echo)
    if (USBUART_bTxIsReady() && Len) {
      USBUART_Write(bReceivedData, Len);		// load packet
      Len = 0;
    }  

	// Read
	Len = USBUART_bGetRxCount();
	if (Len)  {
      USBUART_Read(bReceivedData,Len); 	
      // out to LCD
      LCD_Position(1,LCDPos);      
      LCDPos += Len; 
      if (LCDPos > 15) LCDPos = 0;
      bReceivedData[Len] = '\0';
      LCD_PrString(bReceivedData);         // Print Data  
    }  
  }
}
